export default (relEntryFile: string) => `import { useState, useEffect } from 'react';
import { Models } from 'umi';
import * as app from '@/app';

export type InitialState = Models<'@@initialState'>;
async function getInitialState() {
  if (!app.getInitialState) {
    throw new Error('getInitialState is not defined in ${relEntryFile}');
  }
  return await app.getInitialState();
}

type ThenArg<T> = T extends Promise<infer U> ? U : T

const initState = {
  initialState: undefined as ThenArg<ReturnType<typeof getInitialState>> | undefined,
  loading: true,
  error: undefined as Error | undefined,
}

export default () => {
  const [ state, setState ] = useState(initState);

  const refresh = async() => {
    setState(s => ({ ...s, loading: true, initialState: undefined, error: undefined }))
    let ret = undefined;
    try {
      const asyncFunc = () => new Promise<ReturnType<typeof getInitialState>>(res => res(getInitialState()));
      ret = await asyncFunc();
      setState(s => ({ ...s, initialState: ret, loading: false }));
    } catch(e) {
      setState(s => ({ ...s, error: e, loading: false }));
    }
  };

  useEffect(()=>{
    refresh();
  }, []);

  return {
    ...state,
    refresh,
  }
}
`